# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 Attila Siposs <sipiatti@gmail.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

import gettext, locale, os.path
from gettext import gettext as _
from gi.repository import Gtk, Gio, Gdk, GObject # pylint: disable=E0611
from pynote_lib.helpers import get_data_file, show_uri, get_help_uri

WHEREIAM = os.path.abspath(os.path.dirname(__file__))
locale.setlocale(locale.LC_ALL,'')
if WHEREIAM.startswith("/usr"):
    locale.bindtextdomain('pynote','/usr/share/locale')
elif WHEREIAM.startswith("/opt"):
    locale.bindtextdomain('pynote','/opt/extras.ubuntu.com/pynote/share/locale/')
else:
    locale.bindtextdomain('pynote',os.path.abspath(os.path.join(os.path.dirname(__file__),'../../locale/')))
    gettext.bindtextdomain('pynote',os.path.abspath(os.path.join(os.path.dirname(__file__),'../../locale/')))
gettext.textdomain('pynote')
locale.textdomain('pynote')


import logging
logger = logging.getLogger('pynote')

from pynote.PynoteAbout import AboutPynoteDialog
from pynote_lib.PynoteStorage import PynoteTreeStorage

class EditCategoryDialog(Gtk.Dialog):
    def __init__(self, parent):
        Gtk.Dialog.__init__(self, _("Edit category"), parent, 0,
            (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
             Gtk.STOCK_OK, Gtk.ResponseType.OK))
        label = Gtk.Label(_("Category name"))
        label.set_justify(Gtk.Justification.LEFT)
        label.set_alignment(0,0)
        self.entry = Gtk.Entry()
        self.entry.set_activates_default(True)
        self.set_default_response(Gtk.ResponseType.OK)
        hbox = Gtk.Box()
        hbox.set_orientation(Gtk.Orientation.VERTICAL)
        hbox.pack_start(label,True,False,2)
        hbox.pack_start(self.entry,True,False,2)

        box = self.get_content_area()
        box.add(hbox)
        self.show_all()

class NewCategoryDialog(Gtk.Dialog):
    def __init__(self, parent):
        Gtk.Dialog.__init__(self, _("New category"), parent, 0,
            (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
             Gtk.STOCK_OK, Gtk.ResponseType.OK))
        label = Gtk.Label(_("Category name"))
        label.set_justify(Gtk.Justification.LEFT)
        label.set_alignment(0,0)
        self.entry = Gtk.Entry()
        self.entry.set_activates_default(True)
        self.set_default_response(Gtk.ResponseType.OK)
        hbox = Gtk.Box()
        hbox.set_orientation(Gtk.Orientation.VERTICAL)
        hbox.pack_start(label,True,False,2)
        hbox.pack_start(self.entry,True,False,2)

        box = self.get_content_area()
        box.add(hbox)
        self.show_all()

class NewNoteDialog(Gtk.Dialog):
    def __init__(self,parent,categorylist,activecategory):
        Gtk.Dialog.__init__(self, _("New Note"), parent, 0,(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,Gtk.STOCK_OK, Gtk.ResponseType.OK))
        label1 = Gtk.Label(_("Note title"))
        label2 = Gtk.Label(_("Note category"))
        label1.set_justify(Gtk.Justification.LEFT)
        label2.set_justify(Gtk.Justification.LEFT)
        label1.set_alignment(0,0)
        label2.set_alignment(0,0)
        self.entry = Gtk.Entry()
        self.entry.set_activates_default(True)
        self.combo = Gtk.ComboBox()
        renderer = Gtk.CellRendererText()
        self.combo.pack_start(renderer,True)
        self.combo.add_attribute(renderer,"text",1)
        self.combo.set_model(categorylist)
        self.combo.set_active_iter(activecategory)
        self.set_default_response(Gtk.ResponseType.OK)
        box1 = Gtk.Box()
        box1.set_orientation(Gtk.Orientation.VERTICAL)
        box1.pack_start(label1,True,False,2)
        box1.pack_start(self.entry,True,False,2)
        box2 = Gtk.Box()
        box2.set_orientation(Gtk.Orientation.VERTICAL)
        box2.pack_start(label2,True,False,2)
        box2.pack_start(self.combo,True,False,2)
        hbox = Gtk.Box()
        hbox.set_orientation(Gtk.Orientation.HORIZONTAL)
        hbox.pack_start(box1,True,False,2)
        hbox.pack_start(box2,True,False,2)

        box = self.get_content_area()
        box.add(hbox)
        self.show_all()

class PynoteApp(object):
    def __init__(self):
        self.builder = Gtk.Builder()
        self.builder.set_translation_domain('pynote')
        ui_filename = get_data_file('ui', '%s.ui' % ("pynote",))
        self.builder.add_from_file(ui_filename)

        # create settings object
        self.settings = Gio.Settings("net.launchpad.pynote")
        lastnote = self.settings.get_int("lastnote") # get last note id

        # get gui object references
        self.window = self.builder.get_object("pynote_window")
        self.CategoryTreeView = self.builder.get_object("CategoryTreeView")
        self.CategoryCombo = self.builder.get_object("CategoryCombo")
        self.CategoryComboList = self.builder.get_object("CategoryComboList")
        self.NoteView = self.builder.get_object("NoteView")
        self.NoteBuffer = self.builder.get_object("NoteBuffer")
        self.NoteTitleEntry = self.builder.get_object("NoteTitleEntry")
        self.StatusBar = self.builder.get_object("StatusBar")
        self.TrayIcon = self.builder.get_object("statusicon1")

        # instantiate and init own objects
        self.CategoryTree = PynoteTreeStorage() # Gtk.TreeStore(int,int,str)
        self.TreeRenderer = Gtk.CellRendererText()
        column = Gtk.TreeViewColumn("Category",self.TreeRenderer,text=2)
        column.set_sort_column_id(2)
        self.CategoryTreeView.append_column(column)
        self.CategoryTreeView.set_model(self.CategoryTree)
        self.CategorySelection = self.CategoryTreeView.get_selection()
        self.CategorySelection.connect("changed",self.rowselected)


        combo_renderer_text = Gtk.CellRendererText()

        self.CategoryCombo.pack_start(combo_renderer_text, True)
        self.CategoryCombo.add_attribute(combo_renderer_text, "text", 1)
        self.fill_category_combo()

        self.builder.connect_signals(self)
        self.connect_edit_signals()
        self.iconified = False

        if lastnote>0:
            self.CategoryTreeView.expand_to_path(self.CategoryTree.get_path(self.CategoryTree.notepointers[lastnote]))
            self.CategorySelection.select_iter(self.CategoryTree.notepointers[lastnote])
        else:
            self.CategorySelection.select_iter(self.CategoryTree.catpointers[0])

        self.currentnote_dirty = False
        self.StatusBar.set_text(_("Current note db file: %s") % self.settings.get_string("storagefile"))

    def on_mnu_about_activate(self,widget,data = None):
        """Displays the about dialog"""
        about = AboutPynoteDialog()
        about.window.set_modal(True)
        about.window.set_keep_above(True)
        about.window.set_transient_for(self.window)
        response = about.window.run()
        about.window.destroy()

    def on_mnu_contents_activate(self, widget, data = None):
        show_uri(self.window, "ghelp:%s" % get_help_uri())

    def on_mnu_close_activate(self, widget, data=None):
        """Signal handler for closing the PynoteWindow."""
        self.window.destroy()

    def on_pynote_window_destroy(self, widget, data=None):
        """Called when the PynoteWindow is closed."""
        # Clean up code for saving application state should be added here.
        Gtk.main_quit()

    def rowselected(self,selected):
        """Fills in the note editor fields with the data of the selected item"""
        model, iter = selected.get_selected()
        self.disconnect_edit_signals()
        if iter != None:
            if model[iter][1] == 0:
                self.NoteView.set_sensitive(True)
                self.CategoryCombo.set_sensitive(True)
                self.NoteTitleEntry.set_sensitive(True)
                self.NoteBuffer.set_text(self.CategoryTree.note_contents[model[iter][0]])
                self.NoteTitleEntry.set_text(model[iter][2])
                self.settings.set_int("lastnote",model[iter][0])
                self.CategoryCombo.set_active_iter(self.combopointers[model[iter][3]])
            else:
                self.NoteView.set_sensitive(False)
                self.CategoryCombo.set_sensitive(False)
                self.NoteTitleEntry.set_sensitive(False)
                self.NoteBuffer.set_text('')
                self.NoteTitleEntry.set_text('')
                self.settings.set_int("lastnote",0)
                self.CategoryCombo.set_active_iter(None)
        self.connect_edit_signals()

    def fill_category_combo(self):
        """This method fills up the category combo liststore"""
        self.CategoryComboList.clear()
        self.combopointers={}
        for category in self.CategoryTree.catpointers:
            iter = self.CategoryTree.catpointers[category]
            self.combopointers[self.CategoryTree[iter][0]] = self.CategoryComboList.append([self.CategoryTree[iter][0],self.CategoryTree[iter][2]])

    def on_EditCategoryButton_clicked(self,widget,data=None):
        """Popup the category edit dialog and save the category if need"""
        model, iter = self.CategorySelection.get_selected()
        if iter != None:
            if self.CategoryTree[iter][1] != 0 and self.CategoryTree[iter][0] !=0:
                category_name = self.CategoryTree[iter][2]
                dialog = EditCategoryDialog(self.window)
                dialog.entry.set_text(category_name)
                response = dialog.run()
                new_name = dialog.entry.get_text()
                if response == Gtk.ResponseType.OK and new_name != "":
                    self.CategoryTree.modify_category(iter,new_name)
                    self.fill_category_combo()
                dialog.destroy()

    def on_NewCategoryButton_clicked(self,widget,data=None):
        """Popup the new category dialog and save the new category if any"""
        dialog = NewCategoryDialog(self.window)
        response = dialog.run()
        new_name = dialog.entry.get_text()
        if response == Gtk.ResponseType.OK and new_name != "":
            self.CategoryTree.add_new_category(new_name)
            self.fill_category_combo()

        dialog.destroy()

    def on_DeleteButton_clicked(self,widget,data = None):
        """Deletes the current category or note. Only empty categories will be deleted"""
        model, iter = self.CategorySelection.get_selected()
        itemtype = self.CategoryTree[iter][1]
        if itemtype == 0:
            self.CategoryTree.delete_note(iter)
        else:
            has_children = self.CategoryTree.count_children(self.CategoryTree[iter][0])
            if has_children > 0 :
                dialog = Gtk.MessageDialog(self.window, 0, Gtk.MessageType.ERROR,Gtk.ButtonsType.OK, _("The category is not empty!"))
                dialog.format_secondary_text(_("Please remove the notes from the category first."))
                dialog.run()
                dialog.destroy()
            else:
                self.CategoryTree.delete_category(iter)
                self.fill_category_combo()

    def on_NewNoteButton_clicked(self,widget,data=None):
        """Displays new note dialog and create new note if need"""
        model, iter = self.CategorySelection.get_selected()
        if iter == None:
            activecategory = self.combopointers[0]
        else:
            if model[iter][1] == 0:
                activecategory = self.combopointers[model[iter][3]]
            else:
                activecategory = self.combopointers[model[iter][0]]

        dialog = NewNoteDialog(self.window,self.CategoryComboList,activecategory)
        response = dialog.run()
        new_name = dialog.entry.get_text()
        selected_category = self.CategoryComboList[dialog.combo.get_active_iter()][0]
        if response == Gtk.ResponseType.OK and new_name != "":
            new_iter = self.CategoryTree.add_new_note(new_name,selected_category)
            self.CategoryTreeView.expand_to_path(self.CategoryTree.get_path(new_iter))
            self.CategorySelection.select_iter(new_iter)
        dialog.destroy()

    def on_NoteTitleEntry_changed(self, widget, data = None):
        self.currentnote_dirty = True
        self.save_note_if_dirty(None)

    def on_CategoryCombo_changed(self, widget, data = None):
        self.currentnote_dirty = True
        self.save_note_if_dirty(None)

    def on_NoteBuffer_changed(self, widget, data = None):
        self.currentnote_dirty = True
        self.save_note_if_dirty(None)

    def save_note_if_dirty(self,data):
        """Saving note changes on the fly"""
        if self.currentnote_dirty:
            model, iter = self.CategorySelection.get_selected()
            if iter != None and self.CategoryTree[iter][1] == 0:
                note_id = self.CategoryTree[iter][0]
                self.CategoryTree.save_current_note(note_id,self.NoteTitleEntry.get_text(),self.NoteBuffer.get_text(self.NoteBuffer.get_start_iter(),self.NoteBuffer.get_end_iter(),False),self.CategoryComboList[self.CategoryCombo.get_active_iter()][0])
                self.CategoryTreeView.expand_to_path(self.CategoryTree.get_path(self.CategoryTree.notepointers[note_id]))
                self.CategorySelection.select_iter(self.CategoryTree.notepointers[note_id])
                self.currentnote_dirty = False
        return True

    def connect_edit_signals(self):
        self.title_entry_changed_connection = self.NoteTitleEntry.connect("changed", self.on_NoteTitleEntry_changed)
        self.category_combo_changed_connection = self.CategoryCombo.connect("changed", self.on_CategoryCombo_changed)
        self.note_buffer_changed_connection = self.NoteBuffer.connect("changed", self.on_NoteBuffer_changed)

    def disconnect_edit_signals(self):
        self.NoteTitleEntry.disconnect(self.title_entry_changed_connection)
        self.CategoryCombo.disconnect(self.category_combo_changed_connection)
        self.NoteBuffer.disconnect(self.note_buffer_changed_connection)

    def on_mnu_new_file_activate(self,widget,data=None):
        new_file_dialog = Gtk.FileChooserDialog(_("New pynote file"),self.window,Gtk.FileChooserAction.SAVE,(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,Gtk.STOCK_OK, Gtk.ResponseType.OK))
        new_file_dialog.set_current_name("notes.db")
        new_file_dialog.set_filename(os.path.join(os.path.expanduser('~') ,"notes.db"))
        new_file_dialog.set_do_overwrite_confirmation(True)
        response = new_file_dialog.run()
        if response == Gtk.ResponseType.OK:
            new_file_name = new_file_dialog.get_filename()
            os.remove(new_file_name) # TODO: need try-except block
            self.settings.set_string("storagefile",new_file_name)
            self.StatusBar.set_text(_("Current note db file: %s") % new_file_name)
            self.fill_category_combo()
        new_file_dialog.destroy()

    def on_mnu_open_file_activate(self,widget,data=None):
        open_file_dialog = Gtk.FileChooserDialog(_("Open pynote file"),self.window,Gtk.FileChooserAction.OPEN,(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,Gtk.STOCK_OK, Gtk.ResponseType.OK))
        open_file_dialog.set_filename(self.settings.get_string("storagefile"))
        response = open_file_dialog.run()
        if response == Gtk.ResponseType.OK:
            open_file_name = open_file_dialog.get_filename()
            self.settings.set_string("storagefile",open_file_name)
            self.StatusBar.set_text(_("Current note db file: %s") % open_file_name)
            self.fill_category_combo()
        open_file_dialog.destroy()

    def on_mnu_save_as_activate(self,widget,data=None):
        save_file_dialog = Gtk.FileChooserDialog(_("New pynote file"),self.window,Gtk.FileChooserAction.SAVE,(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,Gtk.STOCK_OK, Gtk.ResponseType.OK))
        save_file_dialog.set_filename(self.settings.get_string("storagefile"))
        save_file_dialog.set_do_overwrite_confirmation(True)
        response = save_file_dialog.run()
        if response == Gtk.ResponseType.OK:
            current_file_name = self.settings.get_string("storagefile")
            save_file_name = save_file_dialog.get_filename()
            os.system("cp %s %s" % (current_file_name,save_file_name)) # TODO: need try-except block
            self.settings.set_string("storagefile",save_file_name)
            self.StatusBar.set_text(_("Current note db file: %s") % save_file_name)
            self.fill_category_combo()
        save_file_dialog.destroy()

    def on_statusicon1_activate(self,widget):
        if self.iconified == True:
            self.iconified = False
            self.window.show()
        else:
            self.iconified = True
            self.window.hide()

    def on_pynote_window_delete_event(self,widget,data):
        self.iconified = True
        self.window.hide()
        return True
